/***************************************************************
* (c) 2002 Lichtenberg Consulting
*   All rights reserved.  Permission granted to freely use and
*   distribute so long as this notice appears in tact.
*
* Program: genericaud.p
* Author : Steven Lichtenberg
* Concept: An expansion of work done by Geoff Crawford and Steven
*          Lichtenberg for dynamic query processing and visualization
* Date   : 07/11/2002
* Purpose: Generic routine to collect web information and perform
*          add/update/delete functionality for any table in the system
* Input parameters: Values are retrieved via get-value statements from the
*          web browser.  Depending on what is being done, the inputs will
*          be different.
* Output parameters: obtains a handle to a temp table and marshalls
*          the temp table through creation data query and visualization
* Tables Used: based on input parameters for table name and keyfield value
* Purpose of routine:  A generic framework for
*          dynamically building a query string, getting the dynamically built
*          temp table and displaying the result in the order required.
*          This routine provides a framework for :
*          1) creating records as required.  This generally assumes a
*              temporary data store with copying/post order processing being
*              handled by a separate routine
*          2) generically generating the query and display of the data back
*               to the user (modified based on the front end employed)
*          3) separation of UI/BL/Data access for use by and UI
*          4) Proper navigation through all portions of update/query/display
* Modified:
****************************************************************/


def var i-mode    as char no-undo.
def var prevcnt   as char no-undo.
def var nextcnt   as char no-undo.
def var maxreturn as int no-undo.
def var returncnt as int no-undo.
def var filnm     as char no-undo.
def var wherecl   as char no-undo.
def var keyfld    as char no-undo.
def var hfieldbuf as handle no-undo.
def var reqnum    as char no-undo.
def var h-procs   as handle no-undo.
DEF var op_hMySuperProcedure AS HANDLE NO-UNDO.

PUBLISH "Getdatapumphandle":U (OUTPUT  op_hMySuperProcedure).
IF NOT VALID-HANDLE( op_hMySuperProcedure) THEN
do:
    run datapumphtml.p persistent set  h-procs.
    session:add-super-procedure(h-procs).
end. /* if not valid-handle( op_hMySuperProcedure) */
{src/web/method/cgidefs.i}
/* receipt of data set based on using a web interface */

assign
    filnm       = trim(get-value("gofile"))
    wherecl     = trim(get-value("andwhere"))
    keyfld      = trim(get-value("filekey"))
    i-mode      = trim(get-value("imode"))
    nextcnt     = trim(get-value("nextcnt"))
    prevcnt     = trim(get-value("prevcnt"))
    maxreturn   = 10
    reqnum      = trim(get-value("recno"))
    returncnt   = if i-mode = "prev"  then int(prevcnt) - maxreturn
        else  int(nextcnt)
    wherecl     = if wherecl = "" then "by " + keyfld else wherecl.
                 /* change as required for your application */
create buffer hfieldbuf for table filnm.

case i-mode:
    when "dodelete" then
    do:
        run deleterec.p(input filnm,
            input "rowid(" + filnm + ") = to-rowid(~'"  + reqnum + "~')") no-error.
        if error-status:error then
            {&out} '<h1> The following error prevented the record from being deleted.</h1>' skip
            '<h2>' error-status:get-message(1) '</h2>'.
   end. /* when "dodelete" */
    when "goadd" then
    do:
       run dispaddupd(input "add",
           input filnm,
           input keyfld,
           input "",
           input hfieldbuf,
           input prevcnt,
           input nextcnt).
       return.
    end. /* when goadd */
    when "doadd" then
    do:
        run doaddupd(input filnm,
            input keyfld,
            input "").
    end. /* when doadd */
    when "update" then
    do:
        run dispaddupd(input "update",
            input filnm,
            input keyfld,
            input reqnum,
            input hfieldbuf,
            input prevcnt,
            input nextcnt).
        return.
    end. /* when goupd */
    when "doupd" then
    do:
        run doaddupd(input filnm,
            input keyfld,
            input reqnum).
    end. /* when doupd */

end case. /* i-mode */
run baserun.p(input filnm,
    input keyfld,
    input wherecl,
    input i-mode,
    input nextcnt,
    input prevcnt,
    input maxreturn,
    input returncnt) no-error.
delete object hfieldbuf no-error.
hfieldbuf = ?.
return.

/**************** internal procedures **************/
procedure validate:
    def input parameter hdbtab as handle no-undo.
    def output parameter omessage as char no-undo.
    
    /* this is a generic place holder for validation. Validation is
    likely to come from database records or custom based on specific 
    application rules.  Place code here for validation routines.
    If there is an error, return the mesage in omessage to show that
    there was a problem.
    */
end procedure
procedure doaddupd:
    def input parameter filnm as char no-undo.
    def input parameter keyfld as char no-undo.
    def input parameter reqnum as char no-undo.

    def var hdatatab   as handle no-undo.
    def var hbuf       as handle no-undo.
    def var hfield     as handle no-undo.
    def var hwebfield  as handle no-undo.
    def var hrecordbuf as handle no-undo.
    def var icnt       as int    no-undo.
    def var fieldname  as char   no-undo.
    def var extentcnt  as int    no-undo.
    def var omessage   as char   no-undo.
    
    create temp-table hdatatab.
    hdatatab:add-new-field("typcd","char").
    hdatatab:add-new-field("fieldnm","char").
    hdatatab:add-new-field("fieldval","char").
    hdatatab:add-new-index("typidx").
    hdatatab:add-index-field("typidx","typcd").
    hdatatab:add-new-index("namidx").
    hdatatab:add-index-field("namidx","fieldnm").
    hdatatab:temp-table-prepare("ttvals").

    create buffer hrecordbuf for table filnm.
    hbuf = hdatatab:default-buffer-handle.
    do icnt = 1 to hrecordbuf:num-fields:
        hfield = hrecordbuf:buffer-field(icnt). 
        if hfield:extent = 0 then
        do:
            hbuf:buffer-create.
            assign
                hwebfield = hbuf:buffer-field("fieldnm")
                hwebfield:buffer-value = hfield:name
                fieldname = hfield:name
                hwebfield = hbuf:buffer-field("fieldval")
                hwebfield:buffer-value = trim(get-value(fieldname)).
        end. /* if hfield:extent = 0 */
        else    /*** need to handle extents gracefully */
        do extentcnt = 1 to hfield:extent:
            hbuf:buffer-create.
            assign
                hwebfield = hbuf:buffer-field("fieldnm") 
                hwebfield:buffer-value = hfield:name + "[" + string(extentcnt) + "]"
                fieldname = hwebfield:buffer-value
                hwebfield = hbuf:buffer-field("fieldval")
                hwebfield:buffer-value = trim(get-value(fieldname)).
        end. /* else */
        message "after create " fieldname ' ' hwebfield:buffer-value ' ' .
    end. /* do icnt = 1 to recordbuf:num-fields */
    /* hook for custom validation goes here */
    run validate(input hdatatab,
        output omessage).
    if omessage <> "" then
    do:
        /* this is where you would put error messages and re-run doadd */
        run dispaddupd(input i-mode,
           input filnm,
           input keyfld,
           input reqnum,
           input hfieldbuf,
           input ?,
           input ?).
       return.
    end. /* if omessage <> " */
    run addupdate.p(input filnm,
        input keyfld,
        input reqnum,
        input hdatatab).
end procedure. /* doaddupd */

procedure dispaddupd:
    def input parameter fromwhere as char no-undo.
    def input parameter filnm     as char no-undo.
    def input parameter keyfld    as char no-undo.
    def input parameter reqnum    as char no-undo.
    def input parameter hfieldbuf as handle no-undo.
    def input parameter prevcnt   as char   no-undo.
    def input parameter nextcnt   as char   no-undo.

    def var icnt as int no-undo.

    def var icnt1 as int no-undo.
    def var httab as handle no-undo.

    output-content-type("text/html").
{&out} '<head> <link rel=STYLESHEET TYPE="text/css" HRef="/base.css"></head><body>'.
    {&out} '<h1>' caps(fromwhere) ' ' caps(filnm) '</h1>'.
    case fromwhere:
        when "update" then
        do:
            run getttab2.p(input filnm,
                input " no-lock where rowid(" + filnm + ") = to-rowid(~'" + reqnum + "~')",
                input "",
                input-output icnt1,
                output httab,
                input ?,
                input ?).
            run visualizetable(input httab,
                input filnm,
                input keyfld).
        end. /* when "update" */
        when "add" then
        do:
            run getttab2.p(input filnm,
                input "",
                input "",
                input-output icnt1,
                output httab,
                input ?,
                input -1).

            run PreVisualizeTable(input httab,
                input filnm,
                input keyfld).
            run VisualizeRecord(input httab,
                input filnm,
                input keyfld).
        end. /* when "Add" */
    end case. /* fromwhere */


    {&out} '<input type="hidden" name="gofile" value="' filnm '">'.
    {&out} '<input type="hidden" name="filekey" value="' keyfld '">'.
    {&out} '<input type="hidden" name="recno" value="' reqnum '">'.
    {&out} '<input type="hidden" name="imode" value="'.
    if fromwhere = "update" then {&out} 'doupd'.
    else {&out} 'doadd'.
    {&out} '">'.
    {&out} '<input type="hidden" name="prevcnt" value="' prevcnt '">'.
    {&out} '<input type="hidden" name="nextcnt" value="' nextcnt '">'.
    {&out} '</form></table></body></html>'.
end procedure. /* dispaddupd */

/***** PreVisualizeTable override ********/

PROCEDURE PreVisualizeTable.
    DEFINE INPUT PARAMETER DataTable AS HANDLE NO-UNDO.
        def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.
    {&out} '<form action=~"genericaud.p~">'.
    {&out} "<table>" skip.
   run super(input datatable,
        input filenm,
        input keyfld).

END. /* PreVisualizeTable */

Procedure PostVisualizeTable:
    DEFINE INPUT PARAMETER DataTable AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.
    {&out} "</table>" skip.
end procedure. /* PostVisualizeTable */
/***** VisualizeTable ****/
PROCEDURE VisualizeTable.


    DEFINE INPUT PARAMETER DataTable AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    DEFINE VARIABLE DataTableQuery AS HANDLE NO-UNDO.
    DEFINE VARIABLE isok AS LOGICAL NO-UNDO.
    define variable hbuf  as handle no-undo.

    /* Create a dynamic query using the handle passed in. */
    run PreVisualizeTable (input DataTable,
        input filenm,
        input keyfld).
    CREATE QUERY DataTableQuery.
    hbuf = datatable:default-buffer-handle.
    isok = DataTableQuery:SET-BUFFERS(hbuf). /* assign the buffer to the query */
    message "set buffer " isok ' ' error-status:get-message(1).
    isok = DataTableQuery:QUERY-PREPARE("for each " + DataTable:NAME + " no-lock"). /* query the entire table */
    message "query prepare " isok.
    isok = DataTableQuery:QUERY-OPEN.  /* open the query */
    message " query open " isok.
    /* Run through each record, calling down one level to visualize each record */

    REPEAT:

        /* Get the next record.  Leave the Procedure if we're done. */

        isok = DataTableQuery:GET-NEXT.
        IF NOT isok THEN RETURN.
        IF DataTableQuery:QUERY-OFF-END THEN RETURN.

        /* Visualize the Record */

        RUN VisualizeRecord(INPUT DataTable,
        input filenm,
        input keyfld).

    END.
    run PostVisualizeTable (input DataTable,
        input filenm,
        input keyfld).
end procedure. /* VisualizeTable */
/***** PreVisualizeRecord ******/

PROCEDURE PreVisualizeRecord.
    DEFINE INPUT PARAMETER RecordHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.
END. /* PreVisualizeRecord */

/***** PreVisualizeField ****/
PROCEDURE VisualizeRecord.


    DEFINE INPUT PARAMETER RecordHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    DEFINE VARIABLE DataField AS HANDLE NO-UNDO.
    DEFINE VARIABLE i         AS INTEGER NO-UNDO.
    define variable hbuf      as handle no-undo.
    RUN PreVisualizeRecord (INPUT RecordHandle,
        input filenm,
        input keyfld).
    hbuf = RecordHandle:default-buffer-handle.

    /* Use the Buffer-Handle to dynamically grab each one of the
       data field handles. Perhaps this could be done more
       efficiently by storing the set of record handles only
       once since the Handles do not change.  However, the
       following is the clearest code. */

    /* This code assumes you've created a temp-table just the
       way you want it, without any extraneous fields.  Create
       a different version of the code below if that's not the case. */
    datafield = hbuf:buffer-field(keyfld).
        RUN VisualizeField (INPUT DataField,
               input filenm,
        input keyfld).
    DO i = 1 TO hbuf:NUM-FIELDS:

        DataField = hbuf:BUFFER-FIELD(i). /* get the handle by the field number, not name */
        if datafield:name = keyfld then next.
        /* Visualize the Field */

        RUN VisualizeField (INPUT DataField,
        input filenm,
        input keyfld).

    END.

    RUN PostVisualizeRecord  (INPUT RecordHandle,
        input filenm,
        input keyfld).

END. /* Visualize Record */

PROCEDURE PreVisualizeField.


    DEFINE INPUT PARAMETER FieldHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    {&out} "<tr><th>".
END. /* PreVisualizeField */

/***** VisualizeField  *****/
PROCEDURE VisualizeField.

    DEFINE INPUT PARAMETER FieldHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    def var icnt as int no-undo.

    RUN PreVisualizeField (INPUT FieldHandle,
        input filenm,
        input keyfld).

    /* Actual Visualization here is a simple Display.
       This isn't really all that useful, but it's
       mainly a demo version of VisualizeField. */
    if fieldhandle:name = "rownum" then return.
    if not valid-handle(fieldhandle) then {&out} "nbsp;".
    if fieldhandle:extent > 0 then
    do icnt = 1 to fieldhandle:extent:
        {&out} fieldhandle:label ' ' icnt '</th><td><input type=~"text~" name=~"' Fieldhandle:NAME '[' icnt ']~" value=~"'.
        if FieldHandle:BUFFER-VALUE[icnt] <> ? then {&out} fieldhandle:buffer-value[icnt].
        else {&out} " " .
        {&out} '~">'.
        if icnt <> fieldhandle:extent then {&out} '</td></tr><tr><th>'.
    end. /* if fieldhandle:extent > ) */
    else
    do:
        {&out} fieldhandle:label '<br>(' fieldhandle:help ')</th><td><input type=~"text~" name=~"' Fieldhandle:NAME '~" value=~"' .
        if fieldhandle:buffer-value <> ? then {&out} FieldHandle:BUFFER-VALUE .
        else {&out} " ".
        {&out} '~">'.
    end. /* else */
    RUN PostVisualizeField (INPUT FieldHandle,
        input filenm,
        input keyfld).

END. /* Visualize Field */
/***** PostVisualizeField ****/
PROCEDURE PostVisualizeField.


    DEFINE INPUT PARAMETER FieldHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    {&out} "</td></tr>".
END. /* PostVisualizeField */

/***** PostVisualizeRecord ****/
PROCEDURE PostVisualizeRecord.


    DEFINE INPUT PARAMETER RecordHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.
    {&out} '<tr><td align="center"><input type="button" name="gorecs" value="Submit" onclick="form.submit();"><input type="reset"></td></tr>'.
 END. /* PostVisualizeRecord */

/***** PostVisualizeTable ****/



