/***************************************************************
* Program: gettab.p
* Author : Steven Lichtenberg
* Date   : 08-02-2000
* Purpose: Generic routine to create a dynamic temp-table and populate it
*          based on table name and where clause passed into the routine.  This
*          will then pass back the records required of the display/dort routine
*          and will allow for the separation of data access and display
* Tables Used: Any or all
* Modified:  12/08/2000 SWL added additional parameter to allow for additional
                            fields to be added to the temp-table
             08/28/2001 SWL added code to manage joins in creation of temp-table.
****************************************************************/

/****** Input parameters *********/

DEFINE INPUT  PARAMETER p-table AS CHARACTER  NO-UNDO.
DEFINE INPUT  PARAMETER p-where AS CHARACTER  NO-UNDO.
define input parameter p-adfield as char      no-undo.
define input-output parameter icnt    as int  no-undo.
DEFINE OUTPUT PARAMETER tthnd as handle no-undo.
def input parameter startcnt as int    no-undo.
define input parameter maxcnt as int    no-undo.

/******** Local Variables ********/
DEFINE VARIABLE h-buff          AS HANDLE     NO-UNDO. /* default buffer handle */
DEFINE VARIABLE h-query         AS HANDLE     NO-UNDO. /* query handle */
DEFINE VARIABLE h-dbtab         AS HANDLE     NO-UNDO. /* table buffer */
define variable h-dbtab2        as handle     no-undo.
define variable h-dbtab3        as handle     no-undo.
def var isok as                 logical       no-undo. /* error processing */
def var v-ttnm                  as char       no-undo. /* temp table name */
def var v-qrystring             as char       no-undo.
def var i                       as int        no-undo.
def var cnt                     as int        no-undo.
def var hfield                  as handle     no-undo.
def var icnt1                   as int        no-undo.
def var qrycnt                  as int        no-undo.
def var vtable                  as char       no-undo.

{src/web/method/cgidefs.i}


v-qrystring = "for each " + entry(1,p-table) + " " + p-where.
create temp-table tthnd.
message v-qrystring " " startcnt ' '.
v-ttnm = "tt" + entry(1,p-table).
do cnt = 1 to num-entries(p-table):
    tthnd:add-fields-from(entry(cnt,p-table)) no-error.  /* define the temp table to be like the table */
end. /* do cnt = 1 to num-entries(p-table): */
if p-adfield <> "" then
do icnt1 = 1 to num-entries(p-adfield):
    tthnd:add-new-field(entry(icnt,p-adfield),entry(icnt + 1,p-adfield)) no-error.
    if error-status:error then message "could not add additional fields".
    icnt = icnt + 1.
end. /* if p-adfield <> "" */
tthnd:add-new-field("rownum","char").
tthnd:temp-table-prepare(v-ttnm) no-error.

/**** pass back the empty temp-table definition if this is an add ********/ 
if maxcnt = -1 then return.

h-buff = tthnd:default-buffer-handle.
create buffer h-dbtab for table entry(1,p-table).  /* dynamic buffer creation */
create query h-query.
isok = h-query:set-buffers(h-dbtab) no-error.     /* assign the buffer to the query */
if num-entries(p-table) > 1
then
do cnt = 2 to num-entries(p-table):
    case cnt:
        when 2 then
        do:
            create buffer h-dbtab2 for table entry(cnt,p-table).
            h-query:add-buffer(h-dbtab2).
        end. /* when 2 */
        when 3 then
        do:
            create buffer h-dbtab3 for table entry(cnt,p-table).
            h-query:add-buffer(h-dbtab3).
        end. /* when 3 */
    end case. /* case cnt */
end. /* if num-entries(p-table) > 1 */
isok = h-query:query-prepare(v-qrystring) no-error. /* assign query string */
if not isok then
do i = 1 to 5:
    {&out} 'not isok - in gettab ' error-status:get-message(i) '<br>'.
    message "not isok " program-name(2) error-status:get-message(i).
    if i = 5 then return.
end.
if not isok then return.
isok = h-query:query-open no-error.                /* open the query */
if not isok then return.
assign
    i = if h-query:num-results = ? then 0 else h-query:num-results
    icnt = icnt + i.
/* if i = 0 then return. */


repeat :                                 /* copy table contents to the temp-table */
    h-query:get-next.
    if h-query:query-off-end then
        leave.
     icnt1 = icnt1 + 1.
     message "counting records " icnt1 ' ' startcnt ' ' maxcnt ' 'startcnt <> ? and startcnt <> 0.
     if startcnt <> ? and startcnt <> 0 then if icnt1 < startcnt then next.
    if maxcnt <> ? then
    do:
        qrycnt = qrycnt + 1.
        if qrycnt > maxcnt then leave.
    end. /* if maxcnt <> ? then */
    message "buffer create ".
    h-buff:buffer-create.
    h-buff:buffer-copy(h-dbtab).
    if num-entries(p-table) > 1 then
    do:
        h-buff:buffer-copy(h-dbtab2).
        if valid-handle(h-dbtab3) then
        h-buff:buffer-copy(h-dbtab3).
    end. /* if num-entries(p-table) > 1 then */
    do cnt = 1 to h-buff:num-fields:
        assign
            hfield = h-buff:buffer-field(cnt)
            hfield:private-data = string(h-dbtab:rowid) no-error.            
            if hfield:name = "rownum" then hfield:buffer-value = string(h-dbtab:rowid).
       if error-status:error then {&out} error-status:get-message(1).
   end. /* do cnt = 1 to h-buff:num-fields */
end.  /* repeat */

return string(icnt).




